@@ -10,11 +10,14 @@
 #include "nodes/primnodes.h"
 #include "parser/parse_expr.h"
 #include "parser/parse_oper.h"
+#include "utils/array.h"
+#include "utils/arrayaccess.h"
 #include "utils/builtins.h"
 #include "utils/datum.h"
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
 #include "utils/syscache.h"
+#include "utils/typcache.h"
 #include "orafce.h"
 #include "builtins.h"
 
@@ -28,6 +31,8 @@ static size_t multiplication = 1;
 
 text *def_locale = NULL;
 
+static Datum ora_greatest_least(FunctionCallInfo fcinfo, bool greater);
+
 PG_FUNCTION_INFO_V1(ora_lnnvl);
 
 Datum
@@ -557,3 +562,106 @@ ora_get_status(PG_FUNCTION_ARGS)
 	PG_RETURN_TEXT_P(cstring_to_text("Production"));
 #endif
 }
+
+PG_FUNCTION_INFO_V1(ora_greatest);
+
+/*
+ * ora_greatest(anyarray) returns anyelement
+ */
+Datum
+ora_greatest(PG_FUNCTION_ARGS)
+{
+	PG_RETURN_DATUM(ora_greatest_least(fcinfo, true));
+}
+
+PG_FUNCTION_INFO_V1(ora_least);
+
+/*
+ * ora_least(anyarray) returns anyelement
+ */
+Datum
+ora_least(PG_FUNCTION_ARGS)
+{
+	PG_RETURN_DATUM(ora_greatest_least(fcinfo, false));
+}
+
+/*
+ * ora_greatest_least(anyarray, bool) returns anyelement
+ * Boolean parameter is true for greatest and false for least.
+ */
+static Datum
+ora_greatest_least(FunctionCallInfo fcinfo, bool greater)
+{
+	Datum		result;
+	Datum		value;
+	ArrayType	*array;
+	ArrayIterator array_iterator;
+	Oid		element_type;
+	Oid		collation = PG_GET_COLLATION();
+	ArrayMetaState *my_extra = NULL;
+	bool	isnull;
+
+	/* caller functions are marked as strict */
+	Assert(!PG_ARGISNULL(0));
+	Assert(!PG_ARGISNULL(1));
+
+	array = PG_GETARG_ARRAYTYPE_P(1);
+	element_type = ARR_ELEMTYPE(array);
+
+	Assert(element_type == get_fn_expr_argtype(fcinfo->flinfo, 0));
+
+	/* fast return */
+	if (array_contains_nulls(array))
+		PG_RETURN_NULL();
+
+	if (fcinfo->flinfo->fn_extra == NULL)
+	{
+		fcinfo->flinfo->fn_extra = MemoryContextAlloc(fcinfo->flinfo->fn_mcxt,
+													  sizeof(ArrayMetaState));
+		my_extra = (ArrayMetaState *) fcinfo->flinfo->fn_extra;
+		my_extra->element_type = ~element_type;
+	}
+
+	if (my_extra->element_type != element_type)
+	{
+		Oid		sortop_oid;
+
+		get_typlenbyvalalign(element_type,
+							 &my_extra->typlen,
+							 &my_extra->typbyval,
+							 &my_extra->typalign);
+
+		if (greater)
+			get_sort_group_operators(element_type, false, false, true,
+									 NULL, NULL, &sortop_oid, NULL);
+		else
+			get_sort_group_operators(element_type, true, false, false,
+									 &sortop_oid, NULL, NULL, NULL);
+
+		my_extra->element_type = element_type;
+
+		fmgr_info_cxt(get_opcode(sortop_oid), &my_extra->proc,
+					  fcinfo->flinfo->fn_mcxt);
+	}
+
+	/* Let's return the first parameter by default */
+	result = PG_GETARG_DATUM(0);
+
+	array_iterator = array_create_iterator(array, 0, my_extra);
+	while (array_iterate(array_iterator, &value, &isnull))
+	{
+		/* not nulls, so run the operator */
+		if (!DatumGetBool(FunctionCall2Coll(&my_extra->proc, collation,
+										   result, value)))
+			result = value;
+	}
+
+	result = datumCopy(result, my_extra->typbyval, my_extra->typlen);
+
+	array_free_iterator(array_iterator);
+
+	/* Avoid leaking memory when handed toasted input */
+	PG_FREE_IF_COPY(array, 1);
+
+	PG_RETURN_DATUM(result);
+}
